# Copyright (C) 2006-2007 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from ricci_communicator import RicciCommunicator
import threading

def resolveOSType(os_str):
	if not os_str or os_str.find('Tikanga') != (-1) or os_str.find('FC6') != (-1) or os_str.find('Zod') != (-1) or os_str.find('Moonshine') != (-1):
		return 'rhel5'
	else:
		return 'rhel4'

def add_commas(self, str1, str2):
	return '%s; %s' % (str1, str2)

class Worker(threading.Thread):
	def __init__(self, mutex, hosts, riccis, func=None, *args):
		threading.Thread.__init__(self)
		self.mutex = mutex
		self.hosts = hosts
		self.riccis = riccis
		self.query_func = func
		self.query_args = args

	def run(self):
		while True:
			self.mutex.acquire()
			if len(self.hosts) == 0:
				self.mutex.release()
				return
			host = self.hosts.pop()
			self.mutex.release()

			r = { 'ricci': None }

			try:
				rc = RicciCommunicator(host)
				r['ricci'] = rc
				try:
					r['cluster_name'] = rc.cluster_info()[0]
				except:
					pass

				if self.query_func is not None:
					if self.query_args:
						args = list(self.query_args)
					else:
						args = list()
					args.insert(0, rc)
					r['batch_result'] = self.query_func(*args)
			except Exception, e:
				pass

			self.mutex.acquire()
			self.riccis[host] = r
			self.mutex.release()

def send_batch_to_hosts(system_list, max_threads, func, *args):
	mutex = threading.RLock()
	threads = list()
	hosts = list()
	num_hosts = 0
	ret = {}

	for host in system_list:
		hosts.append(host)
		num_hosts += 1
		if num_hosts <= max_threads:
			threads.append(Worker(mutex, hosts, ret, func, *args))
	for thread in threads:
		thread.start()
	for thread in threads:
		thread.join()
	return ret

def get_system_info(self, system_list):
	mutex = threading.RLock()
	hive  = [] # workers
	ss	= {} # storage systems (will store riccis and then use them to retrieve real info)
	hosts = [] # hostnames

	for system in system_list:
		hosts.append(system[0])
		if len(hosts) < 10:
			hive.append(Worker(mutex, hosts, ss))

	for bee in hive:
		bee.start()

	for bee in hive:
		bee.join()

	for hostname in ss.keys():
		OS = ''
		cluname = ''
		cluali = ''
		key_fp = ''
		trusted = ''
		authed = False
		trusted = False
		ricci = ss[hostname]['ricci']

		if ricci is not None:
			OS = ricci.os()
			cluname = ricci.cluster_info()[0]
			cluali = ricci.cluster_info()[1]
			authed = ricci.authed()
			key_fp = ricci.fingerprint()
			trusted = ricci.trusted()
		else:
			OS = 'System not accessible'

		s = {
			'hostname'	: hostname,
			'OS'		: OS,
			'cluname'	: cluname,
			'key_fp'	: key_fp,
			'clualias'	: cluali,
			'available'	: ricci is not None,
			'trusted'	: trusted,
			'authed'	: authed
		}

		# replace ricci with system's info
		ss[hostname] = s

	ss_list = []
	sorted_keys = ss.keys()
	sorted_keys.sort()

	for name in sorted_keys:
		ss_list.append(ss[name])
	return ss_list

def convert_bytes(bytes, units):
	c = int(bytes) / get_units_multiplier(units)
	c = repr(c)
	c = c[:c.find('.') + 3]
	return c

# returns (str(float), units) that fits best,
# ignores prefered units

def bytes_to_value_units(bytes):
	units = 'TB'
	if float(convert_bytes(bytes, 'GB')) < 1024:
		units = 'GB'
	if float(convert_bytes(bytes, 'MB')) < 1024:
		units = 'MB'
	if float(convert_bytes(bytes, 'KB')) < 1024:
		units = 'KB'
	if float(convert_bytes(bytes, 'bytes')) < 1024:
		units = 'bytes'
	return (convert_bytes(bytes, units), units)

def get_units_multiplier(units):
	if units.lower() == 'bytes':
		return 1.0
	elif units.lower() == 'kb':
		return 1024.0
	elif units.lower() == 'mb':
		return 1024*1024.0
	elif units.lower() == 'gb':
		return 1024*1024*1024.0
	elif units.lower() == 'tb':
		return 1024*1024*1024*1024.0
	else:
		raise Exception, 'Invalid size unit: %s' % units
